package ballwar.model.strategy;
import java.awt.*;
import util.*;

import ballwar.model.*;

/**
 * Abstract strategy that provides random initialization of a ball.
 */
public abstract class AUpdateStrategy implements IUpdateStrategy {
  /**
  * Randomizer to generate desired random value.
  */
 private final static IRandomizer _Rand = Randomizer.Singleton;
  /**
  * The maximum radius that a ball can have.
  */
 private final static int _MaxRadius = 50;
  /**
  * The minimum radius that a ball can have.
  */
 private final static int _MinRadius = 5;
  /**
  * The maximum speed a ball may have.
  */
 private final static int _MaxSpeed = 20;
  
  /**
  * The minimum and maximum values of a ball's  velocityl.
  */
 private Rectangle _MaxVel = new Rectangle(-_MaxSpeed,-_MaxSpeed, _MaxSpeed, _MaxSpeed);

  /**
  * Abstract behavior relegated to concrete subclass.
  * @param context The ball to update.
  */
 public abstract void updateState(Ball context);
  
  /**
  * Initializes the ball with a random radius, velocity, 
  * location (within the bounds of the ball's environment)  and color.
  * The ball's mass is set to be equal to its area.
  * @param context The ball to initialize.
  */
 public void init(Ball context) {
    context.setRadius(_Rand.randomInt(_MinRadius, _MaxRadius));
    context.setVelocity(_Rand.randomVel(_MaxVel));
    context.setColor(_Rand.randomColor());
    context.setMass(Math.PI*context.getRadius()*context.getRadius());
    int r = context.getRadius();
    context.setLocation(new Point(_Rand.randomInt(r, context.getEnv().getWidth()-r), 
                                  _Rand.randomInt(r, context.getEnv().getHeight()-r)));
  }
  
  /**
  * No-op.
  * @param context not used
  * @param target not used
  */
 public void updateCollision(Ball context, Ball target) {
  }

  /**
  * Calls the context's doKill() method to remove it from the system.
  * @param context The ball to kill.
  */
 public void kill(Ball context) {
    context.doKill();
  }
}