package ballwar.model.strategy;
import java.awt.*;

import ballwar.model.*;
import util.*;

/**
 * ANoOpStrategy-based strategy that continuously modifies the velocity to produce circular motion.
 */
public class CurveStrategy extends ANoOpStrategy {
  /**
	 * Randomizer used to randomly generate the increment angle.
	 */
	private IRandomizer rand = Randomizer.Singleton;
  
  /**
	 * The incremental amount to rotate the context's velocity by each time its updateState is called (in radians).
	 */
	private double angle = rand.randomDouble(-Math.PI/4.0, Math.PI/4.0);
  /**
	 * Precalculated value of cos(angle)
	 */
	private double cosA = Math.cos(angle);
  /**
	 * Pre-calculated value of sin(angle)
	 */
	private double sinA = Math.sin(angle);
  
  
  /**
	 * Rotates the context's velocity by the specified angle.
	 * @param context The strategy's context
	 */
	public void updateState(Ball context) {
    Point v = context.getVelocity();
    v.setLocation(cosA*v.getX()+ sinA*v.getY(), cosA*v.getY()-sinA*v.getX());
  }
}
