package util;

import java.awt.*;

/**
 * A utility class used to generate different types of random values.
 */
public interface IRandomizer {
  /**
   * Generates a random location point subject to the constraint that 0<=X<=maxX and 0<=Y<=maxY.
   */
  public abstract Point randomLoc( Rectangle rect);

  /**
   * Generates a random location point subject to the constraint that 0<=X<=maxX and 0<=Y<=maxY.
   */
  public abstract Point randomLoc( Dimension dim);

  /**
   * Returns a random integer greater than or equal to min and less than or equal to max.
   */
  public abstract int randomInt(int min, int max);

  /**
   * Returns a random double greater than or equal to min and less than or equal to max.
   */
  public abstract double randomDouble(double min, double max);

  /**
   * Returns a random velocity (as a Point) subject to the constraint that the absolute value of the vleocity (speed) is less than maxV.
   */
  public abstract Point randomVel( Rectangle rect);

  /**
   * Returns a random square Dimension, whose width is maxDim.width/2<=width<=maxDim.width
   */
  public abstract Dimension randomDim( Dimension maxDim);

  /**
   * Generates a randomly located and sized rectangle
   */
  public abstract Rectangle randomBounds( Rectangle rect, Dimension maxDim);

  /**
   * Generates a random color
   */
  public abstract Color randomColor();

  /**
   * Returns a random choice of one of two objects, x and y, where probX is the probability 
   * that x will be picked (0<=x<=1).
   */
  public abstract Object randomChoice(Object x, Object y, double probX);
}